@@ -2,6 +2,7 @@
 
 #include "nix/store/local-overlay-store.hh"
 #include "nix/util/callback.hh"
+#include "nix/util/os-string.hh"
 #include "nix/store/realisation.hh"
 #include "nix/util/processes.hh"
 #include "nix/util/url.hh"
@@ -33,9 +34,9 @@ StoreReference LocalOverlayStoreConfig::getReference() const
     };
 }
 
-Path LocalOverlayStoreConfig::toUpperPath(const StorePath & path) const
+std::filesystem::path LocalOverlayStoreConfig::toUpperPath(const StorePath & path) const
 {
-    return upperLayer + "/" + path.to_string();
+    return upperLayer.get() / path.to_string();
 }
 
 LocalOverlayStore::LocalOverlayStore(ref<const Config> config)
@@ -43,13 +44,13 @@ LocalOverlayStore::LocalOverlayStore(ref<const Config> config)
     , LocalFSStore{*config}
     , LocalStore{static_cast<ref<const LocalStore::Config>>(config)}
     , config{config}
-    , lowerStore(openStore(percentDecode(config->lowerStoreUri.get())).dynamic_pointer_cast<LocalFSStore>())
+    , lowerStore(openStore(config->lowerStoreUri.get()).dynamic_pointer_cast<LocalFSStore>())
 {
     if (config->checkMount.get()) {
         std::smatch match;
         std::string mountInfo;
         auto mounts = readFile(std::filesystem::path{"/proc/self/mounts"});
-        auto regex = std::regex(R"((^|\n)overlay )" + config->realStoreDir.get() + R"( .*(\n|$))");
+        auto regex = std::regex(R"((^|\n)overlay )" + config->realStoreDir.get().string() + R"( .*(\n|$))");
 
         // Mount points can be stacked, so there might be multiple matching entries.
         // Loop until the last match, which will be the current state of the mount point.
@@ -58,16 +59,16 @@ LocalOverlayStore::LocalOverlayStore(ref<const Config> config)
             mounts = match.suffix();
         }
 
-        auto checkOption = [&](std::string option, std::string value) {
-            return std::regex_search(mountInfo, std::regex("\\b" + option + "=" + value + "( |,)"));
+        auto checkOption = [&](std::string_view option, const std::filesystem::path & value) {
+            return std::regex_search(mountInfo, std::regex("\\b" + option + "=" + value.string() + "( |,)"));
         };
 
         auto expectedLowerDir = lowerStore->config.realStoreDir.get();
-        if (!checkOption("lowerdir", expectedLowerDir) || !checkOption("upperdir", config->upperLayer)) {
-            debug("expected lowerdir: %s", expectedLowerDir);
-            debug("expected upperdir: %s", config->upperLayer);
+        if (!checkOption("lowerdir", expectedLowerDir) || !checkOption("upperdir", config->upperLayer.get())) {
+            debug("expected lowerdir: %s", PathFmt(lowerStore->config.realStoreDir.get()));
+            debug("expected upperdir: %s", PathFmt(config->upperLayer.get()));
             debug("actual mount: %s", mountInfo);
-            throw Error("overlay filesystem '%s' mounted incorrectly", config->realStoreDir.get());
+            throw Error("overlay filesystem %s mounted incorrectly", PathFmt(config->realStoreDir.get()));
         }
     }
 }
@@ -204,19 +205,18 @@ void LocalOverlayStore::collectGarbage(const GCOptions & options, GCResults & re
     remountIfNecessary();
 }
 
-void LocalOverlayStore::deleteStorePath(const Path & path, uint64_t & bytesFreed)
+void LocalOverlayStore::deleteStorePath(const std::filesystem::path & path, uint64_t & bytesFreed, bool isKnownPath)
 {
-    auto mergedDir = config->realStoreDir.get() + "/";
-    if (path.substr(0, mergedDir.length()) != mergedDir) {
-        warn("local-overlay: unexpected gc path '%s' ", path);
+    if (path.parent_path() != config->realStoreDir.get()) {
+        warn("local-overlay: unexpected gc path %s", PathFmt(path));
         return;
     }
 
-    StorePath storePath = {path.substr(mergedDir.length())};
+    StorePath storePath = {path.filename().string()};
     auto upperPath = config->toUpperPath(storePath);
 
     if (pathExists(upperPath)) {
-        debug("upper exists: %s", path);
+        debug("upper exists: %s", PathFmt(path));
         if (lowerStore->isValidPath(storePath)) {
             debug("lower exists: %s", storePath.to_string());
             // Path also exists in lower store.
@@ -226,7 +226,7 @@ void LocalOverlayStore::deleteStorePath(const Path & path, uint64_t & bytesFreed
         } else {
             // Path does not exist in lower store.
             // So we can delete via overlayfs and not need to remount.
-            LocalStore::deleteStorePath(path, bytesFreed);
+            LocalStore::deleteStorePath(path, bytesFreed, isKnownPath);
         }
     }
 }
@@ -246,7 +246,7 @@ void LocalOverlayStore::optimiseStore()
         if (lowerStore->isValidPath(path)) {
             uint64_t bytesFreed = 0;
             // Deduplicate store path
-            deleteStorePath(toRealPath(path), bytesFreed);
+            deleteStorePath(toRealPath(path), bytesFreed, true);
         }
         done++;
         act.progress(done, paths.size());
@@ -260,7 +260,7 @@ LocalStore::VerificationResult LocalOverlayStore::verifyAllValidPaths(RepairFlag
     StorePathSet done;
 
     auto existsInStoreDir = [&](const StorePath & storePath) {
-        return pathExists(config->realStoreDir.get() + "/" + storePath.to_string());
+        return pathExists((config->realStoreDir.get() / storePath.to_string()).string());
     };
 
     bool errors = false;
@@ -281,9 +281,9 @@ void LocalOverlayStore::remountIfNecessary()
         return;
 
     if (config->remountHook.get().empty()) {
-        warn("'%s' needs remounting, set remount-hook to do this automatically", config->realStoreDir.get());
+        warn("%s needs remounting, set remount-hook to do this automatically", PathFmt(config->realStoreDir.get()));
     } else {
-        runProgram(config->remountHook, false, {config->realStoreDir});
+        runProgram(config->remountHook.get(), false, {config->realStoreDir.get().native()});
     }
 
     _remountRequired = false;
