@@ -11,10 +11,10 @@
 
 namespace nix {
 
-SSHStoreConfig::SSHStoreConfig(std::string_view scheme, std::string_view authority, const Params & params)
+SSHStoreConfig::SSHStoreConfig(const ParsedURL::Authority & authority, const Params & params)
     : Store::Config{params}
     , RemoteStore::Config{params}
-    , CommonSSHStoreConfig{scheme, authority, params}
+    , CommonSSHStoreConfig{authority, params}
 {
 }
 
@@ -97,11 +97,11 @@ MountedSSHStoreConfig::MountedSSHStoreConfig(StringMap params)
 {
 }
 
-MountedSSHStoreConfig::MountedSSHStoreConfig(std::string_view scheme, std::string_view host, StringMap params)
+MountedSSHStoreConfig::MountedSSHStoreConfig(const ParsedURL::Authority & authority, StringMap params)
     : StoreConfig(params)
     , RemoteStoreConfig(params)
-    , CommonSSHStoreConfig(scheme, host, params)
-    , SSHStoreConfig(scheme, host, params)
+    , CommonSSHStoreConfig(authority, params)
+    , SSHStoreConfig(authority, params)
     , LocalFSStoreConfig(params)
 {
 }
@@ -177,12 +177,12 @@ struct MountedSSHStore : virtual SSHStore, virtual LocalFSStore
      * privilege escalation / symlinks in directories owned by the
      * originating requester that they cannot delete.
      */
-    Path addPermRoot(const StorePath & path, const Path & gcRoot) override
+    std::filesystem::path addPermRoot(const StorePath & path, const std::filesystem::path & gcRoot) override
     {
         auto conn(getConnection());
         conn->to << WorkerProto::Op::AddPermRoot;
         WorkerProto::write(*this, *conn, path);
-        WorkerProto::write(*this, *conn, gcRoot);
+        WorkerProto::write(*this, *conn, gcRoot.string());
         conn.processStderr();
         return readString(conn->from);
     }
@@ -208,7 +208,7 @@ ref<RemoteStore::Connection> SSHStore::openConnection()
         command.push_back(config->remoteStore.get());
     }
     command.insert(command.end(), extraRemoteProgramArgs.begin(), extraRemoteProgramArgs.end());
-    conn->sshConn = master.startCommand(std::move(command));
+    conn->sshConn = master.startCommand(toOsStrings(std::move(command)));
     conn->to = FdSink(conn->sshConn->in.get());
     conn->from = FdSource(conn->sshConn->out.get());
     return conn;
