@@ -11,7 +11,7 @@ LocalSigner::LocalSigner(SecretKey && privateKey)
 {
 }
 
-std::string LocalSigner::signDetached(std::string_view s) const
+Signature LocalSigner::signDetached(std::string_view s) const
 {
     return privateKey.signDetached(s);
 }
