@@ -15,6 +15,7 @@
 #include "nix/store/derivations.hh"
 #include "nix/util/finally.hh"
 #include "nix/cmd/legacy.hh"
+#include "nix/cmd/unix-socket-server.hh"
 #include "nix/store/daemon.hh"
 #include "man-pages.hh"
 
@@ -27,8 +28,6 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/un.h>
 #include <sys/select.h>
 #include <errno.h>
 #include <pwd.h>
@@ -39,10 +38,6 @@
 #  include "nix/util/cgroup.hh"
 #endif
 
-#if defined(__APPLE__) || defined(__FreeBSD__)
-#  include <sys/ucred.h>
-#endif
-
 using namespace nix;
 using namespace nix::daemon;
 
@@ -197,63 +192,6 @@ matchUser(const std::optional<std::string> & user, const std::optional<std::stri
     return false;
 }
 
-struct PeerInfo
-{
-    std::optional<pid_t> pid;
-    std::optional<uid_t> uid;
-    std::optional<gid_t> gid;
-};
-
-/**
- * Get the identity of the caller, if possible.
- */
-static PeerInfo getPeerInfo(int remote)
-{
-    PeerInfo peer;
-
-#if defined(SO_PEERCRED)
-
-#  if defined(__OpenBSD__)
-    struct sockpeercred cred;
-#  else
-    ucred cred;
-#  endif
-    socklen_t credLen = sizeof(cred);
-    if (getsockopt(remote, SOL_SOCKET, SO_PEERCRED, &cred, &credLen) == 0) {
-        peer.pid = cred.pid;
-        peer.uid = cred.uid;
-        peer.gid = cred.gid;
-    }
-
-#elif defined(LOCAL_PEERCRED)
-
-#  if !defined(SOL_LOCAL)
-#    define SOL_LOCAL 0
-#  endif
-
-    xucred cred;
-    socklen_t credLen = sizeof(cred);
-    if (getsockopt(remote, SOL_LOCAL, LOCAL_PEERCRED, &cred, &credLen) == 0)
-        peer.uid = cred.cr_uid;
-
-#endif
-
-    return peer;
-}
-
-#define SD_LISTEN_FDS_START 3
-
-/**
- * Open a store without a path info cache.
- */
-static ref<Store> openUncachedStore()
-{
-    Store::Config::Params params; // FIXME: get params from somewhere
-    // Disable caching since the client already does that.
-    params["path-info-cache-size"] = "0";
-    return openStore(settings.storeUri, params);
-}
-
 /**
  * Authenticate a potential client
  *
@@ -265,7 +203,7 @@ static ref<Store> openUncachedStore()
  *
  * If the potential client is not allowed to talk to us, we throw an `Error`.
  */
-static std::pair<TrustedFlag, std::optional<std::string>> authPeer(const PeerInfo & peer)
+static std::pair<TrustedFlag, std::optional<std::string>> authPeer(const unix::PeerInfo & peer)
 {
     TrustedFlag trusted = NotTrusted;
 
@@ -285,7 +223,7 @@ static std::pair<TrustedFlag, std::optional<std::string>> authPeer(const PeerInf
     if (matchUser(user, group, trustedUsers))
         trusted = Trusted;
 
-    if ((!trusted && !matchUser(user, group, allowedUsers)) || group == settings.buildUsersGroup)
+    if ((!trusted && !matchUser(user, group, allowedUsers)) || group == settings.getLocalSettings().buildUsersGroup)
         throw Error("user '%1%' is not allowed to connect to the Nix daemon", user.value_or("<unknown>"));
 
     return {trusted, std::move(user)};
@@ -295,37 +233,21 @@ static std::pair<TrustedFlag, std::optional<std::string>> authPeer(const PeerInf
  * Run a server. The loop opens a socket and accepts new connections from that
  * socket.
  *
+ * @param storeConfig The store configuration to use for opening stores.
  * @param forceTrustClientOpt If present, force trusting or not trusted
  * the client. Otherwise, decide based on the authentication settings
  * and user credentials (from the unix domain socket).
  */
-static void daemonLoop(std::optional<TrustedFlag> forceTrustClientOpt)
+static void daemonLoop(ref<const StoreConfig> storeConfig, std::optional<TrustedFlag> forceTrustClientOpt)
 {
     if (chdir("/") == -1)
         throw SysError("cannot change current directory");
 
-    AutoCloseFD fdSocket;
-
-    //  Handle socket-based activation by systemd.
-    auto listenFds = getEnv("LISTEN_FDS");
-    if (listenFds) {
-        if (getEnv("LISTEN_PID") != std::to_string(getpid()) || listenFds != "1")
-            throw Error("unexpected systemd environment variables");
-        fdSocket = SD_LISTEN_FDS_START;
-        unix::closeOnExec(fdSocket.get());
-    }
-
-    //  Otherwise, create and bind to a Unix domain socket.
-    else {
-        createDirs(dirOf(settings.nixDaemonSocketFile));
-        fdSocket = createUnixDomainSocket(settings.nixDaemonSocketFile, 0666);
-    }
-
     //  Get rid of children automatically; don't let them become zombies.
     setSigChldAction(true);
 
 #ifdef __linux__
-    if (settings.useCgroups) {
+    if (settings.getLocalSettings().useCgroups) {
         experimentalFeatureSettings.require(Xp::Cgroups);
 
         //  This also sets the root cgroup to the current one.
@@ -333,9 +255,9 @@ static void daemonLoop(std::optional<TrustedFlag> forceTrustClientOpt)
         auto cgroupFS = getCgroupFS();
         if (!cgroupFS)
             throw Error("cannot determine the cgroups file system");
-        auto rootCgroupPath = canonPath(*cgroupFS + "/" + rootCgroup);
+        auto rootCgroupPath = *cgroupFS / rootCgroup.rel();
         if (!pathExists(rootCgroupPath))
-            throw Error("expected cgroup directory '%s'", rootCgroupPath);
+            throw Error("expected cgroup directory %s", PathFmt(rootCgroupPath));
         auto daemonCgroupPath = rootCgroupPath + "/nix-daemon";
         //  Create new sub-cgroup for the daemon.
         if (mkdir(daemonCgroupPath.c_str(), 0755) != 0 && errno != EEXIST)
@@ -345,81 +267,67 @@ static void daemonLoop(std::optional<TrustedFlag> forceTrustClientOpt)
     }
 #endif
 
-    //  Loop accepting connections.
-    while (1) {
+    try {
+        unix::serveUnixSocket(
+            {
+                .socketPath = settings.nixDaemonSocketFile,
+                .socketMode = 0666,
+            },
+            [&](AutoCloseFD remote, std::function<void()> closeListeners) {
+                unix::closeOnExec(remote.get());
 
-        try {
-            //  Accept a connection.
-            struct sockaddr_un remoteAddr;
-            socklen_t remoteAddrLen = sizeof(remoteAddr);
+                unix::PeerInfo peer;
+                TrustedFlag trusted;
+                std::optional<std::string> userName;
 
-            AutoCloseFD remote = accept(fdSocket.get(), (struct sockaddr *) &remoteAddr, &remoteAddrLen);
-            checkInterrupt();
-            if (!remote) {
-                if (errno == EINTR)
-                    continue;
-                throw SysError("accepting connection");
-            }
+                if (forceTrustClientOpt)
+                    trusted = *forceTrustClientOpt;
+                else {
+                    peer = unix::getPeerInfo(remote.get());
+                    auto [_trusted, _userName] = authPeer(peer);
+                    trusted = _trusted;
+                    userName = _userName;
+                };
 
-            unix::closeOnExec(remote.get());
+                printInfo(
+                    (std::string) "accepted connection from pid %1%, user %2%" + (trusted ? " (trusted)" : ""),
+                    peer.pid ? std::to_string(*peer.pid) : "<unknown>",
+                    userName.value_or("<unknown>"));
 
-            PeerInfo peer;
-            TrustedFlag trusted;
-            std::optional<std::string> userName;
+                // Fork a child to handle the connection.
+                ProcessOptions options;
+                options.errorPrefix = "unexpected Nix daemon error: ";
+                options.dieWithParent = false;
+                options.runExitHandlers = true;
+                options.allowVfork = false;
+                startProcess(
+                    [&, storeConfig, closeListeners = std::move(closeListeners)]() {
+                        closeListeners();
 
-            if (forceTrustClientOpt)
-                trusted = *forceTrustClientOpt;
-            else {
-                peer = getPeerInfo(remote.get());
-                auto [_trusted, _userName] = authPeer(peer);
-                trusted = _trusted;
-                userName = _userName;
-            };
+                        // Background the daemon.
+                        if (setsid() == -1)
+                            throw SysError("creating a new session");
 
-            printInfo(
-                (std::string) "accepted connection from pid %1%, user %2%" + (trusted ? " (trusted)" : ""),
-                peer.pid ? std::to_string(*peer.pid) : "<unknown>",
-                userName.value_or("<unknown>"));
+                        // Restore normal handling of SIGCHLD.
+                        setSigChldAction(false);
 
-            //  Fork a child to handle the connection.
-            ProcessOptions options;
-            options.errorPrefix = "unexpected Nix daemon error: ";
-            options.dieWithParent = false;
-            options.runExitHandlers = true;
-            options.allowVfork = false;
-            startProcess(
-                [&]() {
-                    fdSocket = -1;
+                        // For debugging, stuff the pid into argv[1].
+                        if (peer.pid && savedArgv[1]) {
+                            auto processName = std::to_string(*peer.pid);
+                            strncpy(savedArgv[1], processName.c_str(), strlen(savedArgv[1]));
+                        }
 
-                    //  Background the daemon.
-                    if (setsid() == -1)
-                        throw SysError("creating a new session");
+                        // Handle the connection.
+                        auto store = storeConfig->openStore();
+                        store->init();
+                        processConnection(store, FdSource(remote.get()), FdSink(remote.get()), trusted, NotRecursive);
 
-                    //  Restore normal handling of SIGCHLD.
-                    setSigChldAction(false);
-
-                    //  For debugging, stuff the pid into argv[1].
-                    if (peer.pid && savedArgv[1]) {
-                        auto processName = std::to_string(*peer.pid);
-                        strncpy(savedArgv[1], processName.c_str(), strlen(savedArgv[1]));
-                    }
-
-                    //  Handle the connection.
-                    processConnection(
-                        openUncachedStore(), FdSource(remote.get()), FdSink(remote.get()), trusted, NotRecursive);
-
-                    exit(0);
-                },
-                options);
-
-        } catch (Interrupted & e) {
-            return;
-        } catch (Error & error) {
-            auto ei = error.info();
-            // FIXME: add to trace?
-            ei.msg = HintFmt("while processing connection: %1%", ei.msg.str());
-            logError(ei);
-        }
+                        exit(0);
+                    },
+                    options);
+            });
+    } catch (Interrupted & e) {
+        return;
     }
 }
 
@@ -481,16 +389,22 @@ static void processStdioConnection(ref<Store> store, TrustedFlag trustClient)
  * Entry point shared between the new CLI `nix daemon` and old CLI
  * `nix-daemon`.
  *
+ * @param storeConfig The store configuration to use for opening stores.
  * @param forceTrustClientOpt See `daemonLoop()` and the parameter with
  * the same name over there for details.
  *
  * @param processOps Whether to force processing ops even if the next
  * store also is a remote store and could process it directly.
  */
-static void runDaemon(bool stdio, std::optional<TrustedFlag> forceTrustClientOpt, bool processOps)
+static void
+runDaemon(ref<StoreConfig> storeConfig, bool stdio, std::optional<TrustedFlag> forceTrustClientOpt, bool processOps)
 {
+    // Disable caching since the client already does that.
+    storeConfig->pathInfoCacheSize = 0;
+
     if (stdio) {
-        auto store = openUncachedStore();
+        auto store = storeConfig->openStore();
+        store->init();
 
         std::shared_ptr<RemoteStore> remoteStore;
 
@@ -507,7 +421,7 @@ static void runDaemon(bool stdio, std::optional<TrustedFlag> forceTrustClientOpt
             // access to those is explicitly not `nix-daemon`'s responsibility.
             processStdioConnection(store, forceTrustClientOpt.value_or(Trusted));
     } else
-        daemonLoop(forceTrustClientOpt);
+        daemonLoop(storeConfig, forceTrustClientOpt);
 }
 
 static int main_nix_daemon(int argc, char ** argv)
@@ -543,7 +457,7 @@ static int main_nix_daemon(int argc, char ** argv)
             return true;
         });
 
-        runDaemon(stdio, isTrustedOpt, processOps);
+        runDaemon(resolveStoreConfig(StoreReference{settings.storeUri.get()}), stdio, isTrustedOpt, processOps);
 
         return 0;
     }
@@ -551,7 +465,7 @@ static int main_nix_daemon(int argc, char ** argv)
 
 static RegisterLegacyCommand r_nix_daemon("nix-daemon", main_nix_daemon);
 
-struct CmdDaemon : Command
+struct CmdDaemon : StoreConfigCommand
 {
     bool stdio = false;
     std::optional<TrustedFlag> isTrustedOpt = std::nullopt;
@@ -561,7 +475,7 @@ struct CmdDaemon : Command
     {
         addFlag({
             .longName = "stdio",
-            .description = "Attach to standard I/O, instead of trying to bind to a UNIX socket.",
+            .description = "Attach to standard I/O, instead of using UNIX socket(s).",
             .handler = {&stdio, true},
         });
 
@@ -616,9 +530,9 @@ struct CmdDaemon : Command
             ;
     }
 
-    void run() override
+    void run(ref<StoreConfig> storeConfig) override
     {
-        runDaemon(stdio, isTrustedOpt, processOps);
+        runDaemon(std::move(storeConfig), stdio, isTrustedOpt, processOps);
     }
 };
 
